# Binary tree

import prelude.
export BinTree bintree_insert BinTreeIterator bintree_sort.


# Interface
declare BinTree: -> ord return;
  Tuple ord EmptyNode -> tree;
  return tree.
  
declare bintree_insert: -> tree elem return;
  tree -> ord root;
  root -> insert _ _;
  insert elem ord -> root;
  Tuple ord root -> tree;
  return tree.

declare BinTreeIterator: -> tree return;
  return -> end yield;
  tree -> _ root;
  root -> _ _ inorder;
  inorder end yield -> end yield;
  end.

declare bintree_collect: -> tree iter return;
  iter (return tree) -> hd tl;
  bintree_insert tree hd -> tree;
  bintree_collect tree tl return.
  

# Private
declare EmptyNode: -> client;
  client
    # Insert
    (-> elem ord parent;
      Node elem EmptyNode EmptyNode -> node;
      parent node)
    # Delete
    (terminate)
    # In order
    (-> end yield parent;
      parent end yield).

    
declare Node: -> elem left right return;
  return -> client;
  client
    # Insert
    (-> ins_elem ord parent;
      order_before ord ins_elem elem
        ( left -> insert _ _;
          insert ins_elem ord -> left;
          Node elem left right -> node;
          parent node)
        ( right -> insert _ _;
          insert ins_elem ord -> right;
          Node elem left right -> node;
          parent node))
    # Delete
    (terminate)
    # In order
    (-> end yield parent;
      left -> _ _ inorder;
      inorder end yield -> end yield;
      yield elem -> end yield;
      right -> _ _ inorder;
      inorder end yield -> end yield;
      parent end yield).


# Demo
print_string "Inserting 7,2,9,5,and 4 into empty tree...";
Order < = -> order;
BinTree order -> tree;
bintree_insert tree 7 -> tree;
bintree_insert tree 2 -> tree;
bintree_insert tree 9 -> tree;
bintree_insert tree 5 -> tree;
bintree_insert tree 4 -> tree;

print_string "In order iterator:";
BinTreeIterator tree -> iter;
for_each iter print_int;

print_string "Collecting list into tree:";
List_of_7 45 -3 17 34 75 2 1 -> list;
bintree_collect tree list -> tree;
BinTreeIterator tree -> iter;
print_list string_of_int iter;
terminate