(************************************************************ 
 Io Types                                                     
   Amalthea, io_types.mli                                     
   Martin Sandin (d97masa@dtek.chalmers.se)                   
 ************************************************************)


(* Abstract syntax tree *)
type io_id = string

type pos = string * int * int

type en_filter =
  | EFall
  | EFlimit of io_id list
  
type io_expr =
  | Elambda of pos * io_id list * io_expr * en_filter
  | Egetvar of pos * io_id * io_id * io_expr * en_filter
  | Eputvar of pos * io_id * io_expr * io_expr * en_filter
  | Eappl of pos * io_id * io_expr list * en_filter
  | Eid of pos * io_id
  | Eint of pos * int
  | Estring of pos * string
  | Eprimitive of io_id
  
type io_decl = pos * io_id * io_expr

type io_import = io_id * (io_id list) option

type io_imports = io_import list

type io_exports = io_id list

type io_ast = io_imports * io_exports * (io_decl list * io_decl list) * io_expr



(* Interpreter *)
type io_assoc = io_id * (io_cont ref)

and io_vassoc = io_id * (io_cont ref ref)

and io_env = io_assoc list * io_vassoc list

and io_cont =
  | Cprim of (io_cont list -> (io_cont * io_cont list))
  | Cexpr of io_expr * io_env * bool
  | Cint of int
  | Cstring of string
  | Cterminate
  | Clocal of io_expr
  | Cexternal of io_id


(* World *)
type io_module_name = string

type io_module_interface = io_module_name * (io_exports * io_imports * io_ast)

type io_module = io_module_name * (io_env * (io_exports * io_imports * io_ast))

type io_world = io_module_name * io_env * io_module list


(* Exceptions *)
exception Io_error of string

type io_interpreter_error =
  | Io_mismatch_error of string * pos * (int * int)
  | Io_undefined_error of io_id

exception Io_interpreter_error of io_interpreter_error

type io_module_error =
  | Io_parser_error of pos
  | Io_lexer_error of pos
  | Io_module_not_found of string * string
  | Io_undefined_export_error of io_id * io_module_name
  | Io_undefined_import_error of io_id * io_module_name
  
exception Io_module_error of io_module_error

exception Io_prim_error of string

exception Amalathea_AST_error