#  Iterator module (iterator.io)
#  Amalthea standard Io library
#  (c)2002 Martin Sandin (msandin@hotmail.com)
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

import prelude.
export
  for_each
  combine
  map
  accumulate
  fold
  collect
  reverse
  Range
  RangeFrom
  is_empty
  length
  take
  iterate
  head
  tail
  append
  concat
  .
  
# Interface
#  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #
  
declare for_each: -> iter yield return;
  iter return -> head next;
  yield head;
  for_each next yield return.
  
declare Range: -> first_n final_n return;
  return -> end yield;
  = first_n final_n end;
  + first_n 1 -> next_n;
  Range next_n final_n -> next;
  yield first_n next.
  
declare RangeFrom: -> first_n return;
  return -> end yield;
  + first_n 1 -> next_n;
  RangeFrom next_n -> next;
  yield first_n next.
  
  
declare combine: -> iter_a iter_b return;
  return -> end yield;
  iter_a end -> head_a next_a;
  iter_b end -> head_b next_b;
  Tuple head_a head_b -> pair;
  combine next_a next_b -> next;
  yield pair next.
  
declare map: -> f iter return;
  return -> end yield;
  iter end -> head next;
  f head -> head;
  map f next -> next_out;
  yield head next_out.
  
declare collect: -> iter return;
  iter (return Nil) -> elem next;
  collect next -> list;
  cons elem list -> list;
  return list.
  
declare fold: -> f start iter return;
  iter (return start) -> head rest;
  f start head -> start;
  fold f start rest;
  return.

declare accumulate: -> f start iter return;
  return -> end yield;
  _accumulate f start iter -> next_out;
  yield start next_out.

declare length: -> iter return;
  fold (-> a b return; + a 1 return) 0 iter;
  return.

declare reverse: -> list return;
  _reverse list Nil return.
  
declare iterate: -> f start return;
  return -> _ yield;
  f start -> follow;
  iterate f follow -> next;
  yield start next.
  
declare take: -> n iter return;
  return -> end yield;
  = 0 n end;
  - n 1 -> n;
  iter end -> head next;
  take n next -> next_out;
  yield head next_out.

  
declare is_empty: -> iter true false;
  iter true -> _ _;
  false.

declare head: -> iter return;
  iter terminate -> hd tl;
  return hd.

declare tail: -> iter return;
  return -> end yield;
  iter end -> hd tl;
  tl end yield.

declare append: -> iter elem return;
  return -> end yield;
  iter (yield elem Nil) -> hd tl;
  append tl elem -> iter;
  yield hd iter.

declare concat: -> iterA iterB return;
  return -> end yield;
  iterA (iterB end -> hd tl; yield hd tl) -> hd tl;
  concat tl iterB -> iter;
  yield hd iter.
  

# Private
#  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #

declare _reverse: -> source dest return;
  source (return dest) -> head tail;
  cons head dest -> dest;
  _reverse tail dest return.

declare _accumulate: -> f start iter return;
  return -> end yield;
  iter end -> head next;
  f start head -> start;
  _accumulate f start next -> next_out;
  yield start next_out.


# Demo
#  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #  #

print_string "Range example (5-9):";
Range 5 9 -> iter;
for_each iter print_int;

print_string "RangeFrom 0- combined with previous:";
RangeFrom 0 -> iter_b;
combine iter_b iter -> iter;
TuplePrinter string_of_int string_of_int -> printer;
for_each iter printer;

print_string "Map addition of tuples over previous:";
map (-> tuple return;tuple -> a b;+ a b -> c;return c) iter -> iter;
for_each iter print_int;

print_string "Folding * over Range 1-7 (fac 6):";
Range 1 7 -> iter;
fold * 1 iter -> r;
print_int r;

print_string "Accumulate with + over Range 1-7:";
Range 1 7 -> iter;
accumulate + 0 iter -> iter;
for_each iter print_int;

print_string "Length of Range 1-7:";
Range 1 7 -> iter;
length iter -> l;
print_int l;

print_string "Take 5 from reverse Range 1-56:";
Range 1 56 -> iter;
reverse iter -> iter;
take 5 iter -> iter;
for_each iter print_int;

print_string "Tail of previous:";
tail iter -> iter;
for_each iter print_int;

print_string "Take 5 from iterate +4 from 0:";
iterate (-> a return; + a 4 return) 0 -> iter;
take 5 iter -> iter;
for_each iter print_int;

print_string "Append 57 to previous:";
append iter 57 -> iter;
for_each iter print_int;

print_string "Concat previous to itself:";
concat iter iter -> iter;
for_each iter print_int;

terminate
